@@ -25,11 +25,12 @@ import           Data.Char
 import           Data.List              (groupBy, sortBy)
 import           Data.Monoid            (Any (..))
 import           Data.Ord               (comparing)
+import           Data.Text              (Text)
 import qualified Data.Text              as T
 import           Text.CSL.Eval
 import           Text.CSL.Proc.Disamb
 import           Text.CSL.Style         hiding (Any)
-import           Text.CSL.Util          (orIfNull, proc, proc', query)
+import           Text.CSL.Util          (proc, proc', query)
 import           Text.Pandoc.Definition (Inline (Str))
 
 -- | Collapse citations according to the style options.
@@ -38,14 +39,14 @@ collapseCitGroups s
     = map doCollapse
     where
       doCollapse = case getCollapseOptions s of
-                     "year"               : _ -> collapseYear s []
+                     "year"               : _ -> collapseYear s ""
                      "year-suffix"        : _ -> collapseYear s "year-suffix"
                      "year-suffix-ranged" : _ -> collapseYear s "year-suffix-ranged"
                      "citation-number"    : _ -> collapseNumber
                      _                        -> id
 
 -- | Get the collapse option set in the 'Style' for citations.
-getCollapseOptions :: Style -> [String]
+getCollapseOptions :: Style -> [Text]
 getCollapseOptions
     = map snd . filter ((==) "collapse" . fst) . citOptions . citation
 
@@ -103,13 +104,15 @@ getYearAndSuf x
           | OStatus  {} : _ <- o = [head o]
           | otherwise = []
 
-collapseYear :: Style -> String -> CitationGroup -> CitationGroup
-collapseYear s ranged (CG cs f d os) = CG cs f [] (process os)
+collapseYear :: Style -> Text -> CitationGroup -> CitationGroup
+collapseYear s ranged (CG cs f d os) = CG cs f "" (process os)
     where
       styleYSD    = getOptionVal "year-suffix-delimiter"    . citOptions . citation $ s
-      yearSufDel  = styleYSD `orIfNull` (layDelim . citLayout . citation $ s)
+      yearSufDel  = if T.null styleYSD
+                    then layDelim . citLayout . citation $ s
+                    else styleYSD
       afterCD     = getOptionVal "after-collapse-delimiter" . citOptions . citation $ s
-      afterColDel = afterCD  `orIfNull` d
+      afterColDel = if T.null afterCD then d else afterCD
 
       format []     = []
       format (x:xs) = x : map getYearAndSuf xs
@@ -118,14 +121,14 @@ collapseYear s ranged (CG cs f d os) = CG cs f [] (process os)
                    "year-suffix-ranged" -> True
                    _                    -> False
 
-      collapseRange = if null ranged then map (uncurry addCiteAffixes)
+      collapseRange = if T.null ranged then map (uncurry addCiteAffixes)
                       else collapseYearSuf isRanged yearSufDel
 
       rmAffixes x = x {citePrefix = mempty, citeSuffix = mempty}
       delim = let d' = getOptionVal "cite-group-delimiter" . citOptions . citation $ s
               -- FIXME: see https://bitbucket.org/bdarcus/citeproc-test/issue/15
               -- in  if null d' then if null d then ", " else d else d'
-              in  if null d' then ", " else d'
+              in  if T.null d' then ", " else d'
 
       collapsYS a = case a of
                       []  -> (emptyCite, ONull)
@@ -149,10 +152,10 @@ collapseYear s ranged (CG cs f d os) = CG cs f [] (process os)
                          namesOf (snd a) == namesOf (snd b)
       process = doCollapse . groupBy hasSameNames . groupCites
 
-collapseYearSuf :: Bool -> String -> [(Cite,Output)] -> [Output]
+collapseYearSuf :: Bool -> Text -> [(Cite,Output)] -> [Output]
 collapseYearSuf ranged ysd = process
     where
-      yearOf  = concat . query getYear
+      yearOf  = T.concat . query getYear
       getYear o
           | OYear y _ _ <- o = [y]
           | otherwise        = []
@@ -168,8 +171,8 @@ collapseYearSuf ranged ysd = process
                  checkAffix (citeSuffix $ fst a) &&
                  checkAffix (citePrefix $ fst b) &&
                  checkAffix (citeSuffix $ fst b) &&
-                 null (citeLocator $ fst a) &&
-                 null (citeLocator $ fst b)
+                 T.null (citeLocator $ fst a) &&
+                 T.null (citeLocator $ fst b)
 
       getYS []     = []
       getYS [x] = return $ uncurry addCiteAffixes x
@@ -187,17 +190,17 @@ collapseYearSufRanged :: [Output] -> [Output]
 collapseYearSufRanged = process
     where
       getOYS o
-          | OYearSuf s _ _ f <- o = [(if s /= [] then ord (head s) else 0, f)]
+          | OYearSuf s _ _ f <- o = [(if s /= "" then ord (T.head s) else 0, f)]
           | otherwise             = []
       sufOf   = foldr const (0,emptyFormatting) . query getOYS
       newSuf  = map sufOf >>> (map fst >>> groupConsec) &&& map snd >>> uncurry zip
       process xs = flip concatMap (newSuf xs) $
                    \(x,f) -> if length x > 2
-                             then return $ Output [ OStr [chr $ head x] f
+                             then return $ Output [ OStr (T.singleton . chr $ head x) f
                                                   , OPan [Str "\x2013"]
-                                                  , OStr [chr $ last x] f
+                                                  , OStr (T.singleton . chr $ last x) f
                                                   ] emptyFormatting
-                             else map (\y -> if y == 0 then ONull else flip OStr f . return . chr $ y) x
+                             else map (\y -> if y == 0 then ONull else flip OStr f . T.singleton . chr $ y) x
 
 addCiteAffixes :: Cite -> Output -> Output
 addCiteAffixes c x =
